from uvhttp.http import Session
import asyncio
import json

class HueException(Exception):
    pass

class Hue(Session):
    """
    Hue client for :mod:`uvhttp`.
    """
    def __init__(self, loop, hue_api, **kwargs):
        self.__lights = None
        self.__hue_id = None
        self.hue_api = hue_api
        super().__init__(10, loop, **kwargs)

    @property
    def hue_api(self):
        """
        Return the Hue API to use for requests.
        """
        return self.__hue_api

    @hue_api.setter
    def hue_api(self, hue_api):
        """
        Set the Hue API server to use.
        """
        if not hue_api.endswith(b'/'):
            hue_api = hue_api + b'/'
        hue_api = hue_api + b'api'

        self.__hue_api = hue_api

    @property
    def hue_id(self):
        """
        The hue id that will be used when interacting with the API.
        """
        if self.__hue_id:
            return self.__hue_id.decode()

    @hue_id.setter
    def hue_id(self, hue_id):
        """
        Set the hue id for the API to use.
        """
        self.__hue_id = hue_id.encode()

    async def link(self):
        """
        Link with a hue bridge. Return a user id that can be used.
        """
        response = await self.post(self.hue_api, data=json.dumps({
            "devicetype": "uvhue#device"
        }).encode())

        if response.status_code != 200:
            raise HueException('Bad status: {}'.format(response.status_code))

        link_response = response.json()

        if not isinstance(link_response, list):
            raise HueException('Bad JSON returned: {} {}'.format(response.text))

        if len(link_response) < 1:
            raise HueException('Bad JSON returned: {}'.format(response.text))

        if 'success' not in link_response[0]:
            raise HueException('Hue API returned an error: {}'.format(link_response[0]['error']['description']))

        self.hue_id = link_response[0]['success']['username']
        return self.hue_id

    async def api(self, method, path, *args, **kwargs):
        """
        Make an API request to hue with the given path.

        The path should be everything except for the common part of the URL.

        For example, ``https://hue/api/1209310239/lights`` becomes
        ``/lights``.
        """
        url = self.hue_api + b'/' + self.hue_id.encode() + b'/' + path
        return await self.request(method, url, *args, **kwargs)

    async def lights(self, refresh=False):
        """
        Return all of the lights in the system. It will be served from
        cache, if available, if the refresh parameter is not False.
        """
        if not self.__lights or refresh:
            response = await self.api(b'GET', b'lights')
            self.__lights = response.json()

        return self.__lights

    async def set_state(self, light, state):
        """
        Set a state for a light.
        """
        data = json.dumps(state).encode()

        response = await self.api(b'PUT', 'lights/{}/state'.format(light).encode(), data=data)

        return response.json() == state

    async def set_states(self, state, refresh=False):
        """
        Set a state for all lights.
        """
        lights = await self.lights(refresh=refresh)

        state_coros = []
        for light in lights:
           state_coros.append(self.set_state(light, state))

        state_results, _ = await asyncio.wait(state_coros)
        result = True
        for state_result in state_results:
           if not state_result:
               result = False

        return result



from nose.tools import *
from uvhttp.utils import HttpServer
from sanic.response import json
import os
import hashlib

class HueServer(HttpServer):
    def press_button(self):
        self.is_pressed = True

    def add_user(self):
        user = hashlib.md5(os.urandom(8)).hexdigest()
        self.valid_users.append(user)
        return user

    def add_routes(self):
        super().add_routes()

        self.is_pressed = False
        self.valid_users = []

        self.lights = {
            "1": {
                "state": {
                    "on": True,
                    "bri": 144,
                    "hue": 13088,
                    "sat": 212,
                    "xy": [0.5128,0.4147],
                    "ct": 467,
                    "alert": "none",
                    "effect": "none",
                    "colormode": "xy",
                    "reachable": True
                },
                "type": "Extended color light",
                "name": "Hue Lamp 1",
                "modelid": "LCT001",
                "swversion": "66009461",
                "pointsymbol": {
                    "1": "none",
                    "2": "none",
                    "3": "none",
                    "4": "none",
                    "5": "none",
                    "6": "none",
                    "7": "none",
                    "8": "none"
                }
            },
            "2": {
                "state": {
                    "on": False,
                    "bri": 0,
                    "hue": 0,
                    "sat": 0,
                    "xy": [0,0],
                    "ct": 0,
                    "alert": "none",
                    "effect": "none",
                    "colormode": "hs",
                    "reachable": True
                },
                "type": "Extended color light",
                "name": "Hue Lamp 2",
                "modelid": "LCT001",
                "swversion": "66009461",
                "pointsymbol": {
                    "1": "none",
                    "2": "none",
                    "3": "none",
                    "4": "none",
                    "5": "none",
                    "6": "none",
                    "7": "none",
                    "8": "none"
                }
            }
        }

        self.app.add_route(self.link, '/api', methods=['POST'])
        self.app.add_route(self.get_lights, '/api/<user>/lights')
        self.app.add_route(self.set_state, '/api/<user>/lights/<light>/state', methods=['PUT'])

    def link(self, request):
        assert_in('devicetype', request.json)
        if not self.is_pressed:
            return json([
                {
                    "error": {
                        "type":101, "address": "", "description": "link button not pressed"
                    }
                }
            ])
        else:
            self.is_pressed = False
            return json([
                {
                    "success": {
                        "username": self.add_user()
                    }
                }
            ])

    def get_lights(self, request, user):
        if user not in self.valid_users:
            return json([
                {
                    "error": {
                        "type": 1,
                        "address": "/",
                        "description": "unauthorized user"
                    }
                }
            ])
       
        return json(self.lights)

    def set_state(self, request, user, light):
        if user not in self.valid_users:
            return json([
                {
                    "error": {
                        "type": 1,
                        "address": "/",
                        "description": "unauthorized user"
                    }
                }
            ])

        self.lights[light]["state"].update(request.json)
        return json(request.json)



from setuptools import setup
import os

requirements = os.path.join(os.path.dirname(__file__), 'requirements.txt')

setup(
    name='uvhue',
    version='0.4',
    description='Hue client for uvhttp',
    url='https://github.com/justinbarrick/uvhue',
    packages=['uvhue'],
    install_requires=[ r.rstrip() for r in open(requirements).readlines() ]
)



def gamma_correct(color):
    """
    Gamma correct a single color in your rgb tuple.
    """
    if color > 0.04045:
        return pow((color + 0.055) / 1.055, 2.4)
    else:
        return color / 12.92

def gamma_correct_rgb(rgb):
    """
    Gamma correct rgb colors.
    """
    return tuple([ gamma_correct(x) for x in rgb ])

def rgb_to_decimal(rgb):
    """
    Convert rgb format to decimal.
    """
    return tuple([ (x / 255.0) for x in rgb ])

def decimal_to_xyz(rgb):
    """
    Convert decimal format to xyz
    """
    red, green, blue = rgb[0], rgb[1], rgb[2]

    x = red * 0.649926 + green * 0.103455 + blue * 0.197109
    y = red * 0.234327 + green * 0.743075 + blue * 0.022598
    z = red * 0.0000000 + green * 0.053077 + blue * 1.035763

    return (x, y, z)

def xyz_to_xy(xyz):
    """
    Convert xyz tuple to xy
    """
    total = sum(xyz)
    return xyz[0] / total, xyz[1] / total

def rgb_to_xy(rgb):
    """
    Convert a tuple of rgb colors to xy.
    """
    decimal = rgb_to_decimal(rgb)
    decimal = gamma_correct_rgb(decimal)

    xyz = decimal_to_xyz(decimal)
    return xyz_to_xy(xyz)

def is_grey(rgb, threshold=20):
    """
    Return true if an rgb color is a shade of grey.
    """
    return abs(rgb[0] - rgb[1]) <= threshold and abs(rgb[1] - rgb[2]) <= threshold and abs(rgb[2] - rgb[0]) <= threshold
